package ru.javakira.simbirshare.account;

import jakarta.persistence.*;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import ru.javakira.simbirshare.directory.Directory;

import java.util.Collection;
import java.util.List;

@Data
@Entity
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class Account implements UserDetails {
    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE)
    private long id;
    private long diskSize;
    @Column(unique = true)
    private String email; //todo нужна будет валидация
    private String password;
    @Builder.Default
    @Enumerated(EnumType.STRING)
    private AccountState state = AccountState.normal;
    @Builder.Default
    @Enumerated(EnumType.STRING)
    private AccountRole role = AccountRole.user;
    @OneToOne
    private Directory root;

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return List.of(role);
    }

    @Override
    public String getPassword() {
        return password;
    }

    @Override
    public String getUsername() {
        return email;
    }

    @Override
    public boolean isAccountNonExpired() {
        return state != AccountState.deleted;
    }

    @Override
    public boolean isAccountNonLocked() {
        return state != AccountState.deleted;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return state != AccountState.deleted;
    }

    @Override
    public boolean isEnabled() {
        return state != AccountState.deleted;
    }
}
