package ru.javakira.simbirshare.account;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import ru.javakira.simbirshare.directory.Directory;
import ru.javakira.simbirshare.directory.DirectoryRepository;
import ru.javakira.simbirshare.directory.DirectoryService;
import ru.javakira.simbirshare.jwt.JwtService;

@RestController
@RequestMapping("/api/v1/Account")
@RequiredArgsConstructor
class AccountController {
    private final AccountService service;
    private final AccountRepository repository;
    private final DirectoryRepository directoryRepository;
    private final JwtService jwtService;

    @GetMapping("/Me")
    @SecurityRequirement(name = "Bearer Authentication")
    @Operation(summary = "получение данных о текущем аккаунте")
    AccountDto me(HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.accountDto(userId);
    }

    @PostMapping("/SignUp")
    @Operation(summary = "регистрация нового аккаунта")
    AuthResponse signUp(@RequestBody RegisterRequest request) {
        AuthResponse response = service.signUp(request);

        Account account = service.account(response.getAccountId());
        Directory directory = Directory.builder().ownerId(account.getId()).name("user" + account.getId()).build();
        account.setRoot(directory);
        directoryRepository.save(directory);
        repository.save(account);

        return response;
    }

    @PostMapping("/SignIn")
    @Operation(summary = "получение нового jwt токена пользователя")
    AuthResponse signIn(@RequestBody AuthRequest request) {
        return service.signIn(request);
    }

    @PatchMapping("/Update")
    @SecurityRequirement(name = "Bearer Authentication")
    @Operation(summary = "обновление своего аккаунта")
    void update(@RequestBody UpdateRequest updateRequest, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        service.update(userId, updateRequest);
    }

    @PostMapping("/Logout")
    @SecurityRequirement(name = "Bearer Authentication")
    @Operation(summary = "выход из аккаунта")
    void logout(HttpServletRequest request) {
        service.logout(jwtService.token(request));
    }
}
