package ru.javakira.simbirshare.account.admin;

import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import ru.javakira.simbirshare.account.AccountDto;
import ru.javakira.simbirshare.account.AccountService;

import java.util.List;

@Secured("ROLE_ADMIN")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Admin")
@SecurityRequirement(name = "Bearer Authentication")
class AdminAccountController {
    private final AdminAccountService adminService;
    private final AccountService service;

    @GetMapping("/Account")
    List<AccountDto> accounts(AccountsRequest request) {
        return adminService.accounts(request);
    }

    @GetMapping("/Account/{userId}")
    AccountDto accountDto(@PathVariable long userId) {
        return service.accountDto(userId);
    }

    @PostMapping("/Account")
    AccountDto create(AccountCreateRequest request) {
        return adminService.create(request);
    }

    @PutMapping("/Account/{userId}")
    AccountDto put(@PathVariable long userId, AccountPutRequest request) {
        return adminService.put(request, userId);
    }

    @DeleteMapping("/Account/{userId}")
    void delete(@PathVariable long userId) {
        adminService.delete(userId);
    }
}
