package ru.javakira.simbirshare.directory;

import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.servlet.http.HttpServletRequest;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;
import ru.javakira.simbirshare.jwt.JwtService;

import java.util.List;

@RestController
@RequestMapping("/api/v1/Directory")
@RequiredArgsConstructor
@SecurityRequirement(name = "Bearer Authentication")
class DirectoryController {
    private final DirectoryService service;
    private final JwtService jwtService;

    @GetMapping("/GetRoot")
    DirectoryDto getRoot(HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.getRoot(userId);
    }

    @GetMapping("/Directory/{id}")
    DirectoryDto directoryDto(@PathVariable long id, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.directoryDto(userId, id);
    }

    @GetMapping("/GetByDirectory/{id}")
    List<DirectoryDto> getByDirectory(@PathVariable long id, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.getByDirectory(userId, id);
    }

    @PostMapping("/Directory")
    DirectoryDto createDirectory(@RequestBody CreateDirectoryRequest createDirectoryRequest, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.createDirectory(userId, createDirectoryRequest);
    }

    @PutMapping("/Directory/{id}")
    DirectoryDto putDirectory(@PathVariable long id, @RequestBody PutDirectoryRequest putDirectoryRequest, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.putDirectory(id, putDirectoryRequest, userId);
    }

    @DeleteMapping("/Directory/{id}")
    void delete(@PathVariable long id, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        service.delete(id, userId);
    }
}
