package ru.javakira.simbirshare.directory.admin;

import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import ru.javakira.simbirshare.directory.DirectoryDto;
import ru.javakira.simbirshare.directory.DirectoryService;

import java.util.List;

@Secured("ROLE_ADMIN") /* TODO супер важный вопрос, а если директория не будет иметь parentId? Получачется это опциональное поле, а хотя корневая только одна у пользователя может быть */
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Admin/Directory")
@SecurityRequirement(name = "Bearer Authentication")
class AdminDirectoryController {
    private final AdminDirectoryService adminService;
    private final DirectoryService service;

    @GetMapping("/GetRoot/{userId}")
    DirectoryDto getRoot(@PathVariable long userId) {
        return service.getRoot(userId);
    }

    @GetMapping("/Directory/{id}")
    DirectoryDto getDirectory(@PathVariable long id) {
        return service.directoryDto(id);
    }

    @GetMapping("/GetByDirectory/{dirId}")
    List<DirectoryDto> getByDirectory(@PathVariable long dirId) {
        return service.getByDirectory(dirId);
    }

    @PostMapping
    DirectoryDto create(CreateDirectoryAdminRequest request) {
        return adminService.create(request);
    }

    @DeleteMapping("/{id}")
    void delete(@PathVariable long id) {
        service.delete(id);
    }
}
