package ru.javakira.simbirshare.file;

import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.servlet.http.HttpServletRequest;
import lombok.AllArgsConstructor;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;
import ru.javakira.simbirshare.directory.Directory;
import ru.javakira.simbirshare.directory.DirectoryService;
import ru.javakira.simbirshare.jwt.JwtService;

import java.util.List;

@Service
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/v1/Files")
@SecurityRequirement(name = "Bearer Authentication")
class FileController {
    private final FileService service;
    private final DirectoryService directoryService;
    private final JwtService jwtService;

    @GetMapping("/{id}")
    FileDto fileDto(@PathVariable long id, HttpServletRequest request, FileRequest fileRequest) {
        long userId = jwtService.extractId(request);
        File file = service.file(id);
        Directory directory = directoryService.directory(file.getDirectoryId());
        if (directory.getOwnerId() != userId && !file.isShared())
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access for file denied"
            );

        //todo пароль файлов тоже можно закодировать
        if (directory.getOwnerId() != userId && file.getPassword().isPresent()) {
            if (fileRequest.getPassword().isEmpty())
                throw new ResponseStatusException(
                        HttpStatus.FORBIDDEN,
                        "Password required"
                );

            if (!fileRequest.getPassword().get().equals(file.getPassword().get()))
                throw new ResponseStatusException(
                        HttpStatus.FORBIDDEN,
                        "Wrong password for file"
                );
        }

        return FileDto.from(file);
    }

    @GetMapping("/GetByDirectory/{dirId}")
    List<FileDto> getByDirectory(@PathVariable long dirId, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        Directory directory = directoryService.directory(dirId);
        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access for directory denied"
            );

        return service
                .getByDirectory(dirId)
                .stream()
                .map(FileDto::from)
                .toList();
    }

    @PostMapping //todo нельзя создать два одинаковых файла
    FileDto createFile(@RequestBody NewFileRequest newFileRequest, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        Directory directory = directoryService.directory(newFileRequest.getDirectoryId());
        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access for directory denied"
            );

        return service.createFile(newFileRequest);
    }

    @PutMapping("/{id}")
    FileDto putFile(@RequestBody NewFileRequest putFileRequest, HttpServletRequest request, @PathVariable long id) {
        long userId = jwtService.extractId(request);
        Directory directory = directoryService.directory(putFileRequest.getDirectoryId());
        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access for directory denied"
            );

        return service.putFile(putFileRequest, id);
    }

    @DeleteMapping("/{id}")
    void delete(@PathVariable long id, HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        File file = service.file(id);
        Directory directory = directoryService.directory(file.getDirectoryId());
        if (directory.getOwnerId() != userId)
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access for directory denied"
            );

        service.delete(file.getId());
    }
}
