package ru.javakira.simbirshare.file;

import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;

@Service
@RequiredArgsConstructor
public class FileServiceImpl implements FileService {
    private final FileRepository repository;

    @Override
    public File file(long id) {
        return repository.findById(id).orElseThrow(() -> new ResponseStatusException(
                HttpStatus.NOT_FOUND,
                "File with id %d doesnt exist".formatted(id)
        ));
    }

    @Override
    public List<File> getByDirectory(long dirId) {
        return repository
                .findAll()
                .stream()
                .filter(file -> file.getDirectoryId() == dirId)
                .toList();
    }

    @Override
    public FileDto createFile(NewFileRequest request) {
        if (getByDirectory(request.getDirectoryId()).stream().anyMatch(file -> file.getName().equals(request.getName())))
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST,
                    "File %s arleady exist in dir %d".formatted(request.getName(), request.getDirectoryId())
            );

        File file = File
                .builder()
                .directoryId(request.getDirectoryId())
                .shared(request.isShared())
                .password(request.getPassword().orElse(null))
                .name(request.getName())
                .build();

        repository.save(file);
        return FileDto.from(file);
    }

    @Override
    public FileDto putFile(NewFileRequest request, long id) {
        File file = file(id);
        file.setPassword(request.getPassword().orElse(null));
        file.setName(request.getName());
        file.setDirectoryId(request.getDirectoryId());
        file.setShared(request.isShared());

        repository.save(file);
        return FileDto.from(file);
    }

    @Override
    public void delete(long id) {
        File file = file(id);
        repository.delete(file);
    }
}
