package ru.javakira.simbirshare.file.admin;

import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.annotation.Secured;
import org.springframework.web.bind.annotation.*;
import ru.javakira.simbirshare.file.FileDto;
import ru.javakira.simbirshare.file.FileService;
import ru.javakira.simbirshare.file.NewFileRequest;

import java.util.List;

@Secured("ROLE_ADMIN")
@RestController
@RequestMapping("/api/v1/Admin/Files")
@RequiredArgsConstructor
@SecurityRequirement(name = "Bearer Authentication")
class AdminFileController {
    private final FileService service;

    @GetMapping("/{id}")
    FileDto fileDto(@PathVariable long id) {
        return FileDto.from(service.file(id));
    }

    @GetMapping("/GetByDirectory/{dirId}")
    List<FileDto> getByDirectory(@PathVariable long dirId) {
        return service
                .getByDirectory(dirId)
                .stream()
                .map(FileDto::from)
                .toList();
    }

    @PostMapping
    FileDto newFile(@RequestBody NewFileRequest request) {
        return service.createFile(request);
    }

    @PutMapping("/{id}")
    FileDto putFile(@RequestBody NewFileRequest request, @PathVariable long id) {
        return service.putFile(request, id);
    }

    @DeleteMapping("/{id}")
    void delete(@PathVariable long id) {
        service.delete(id);
    }
}
