package ru.javakira.simbirshare.storage;

import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.server.ResponseStatusException;
import ru.javakira.simbirshare.directory.Directory;
import ru.javakira.simbirshare.directory.DirectoryService;
import ru.javakira.simbirshare.file.File;
import ru.javakira.simbirshare.file.FileRequest;
import ru.javakira.simbirshare.file.FileService;
import ru.javakira.simbirshare.jwt.JwtService;

@RestController
@RequestMapping("/api/v1/Storage")
@RequiredArgsConstructor
@SecurityRequirement(name = "Bearer Authentication")
class StorageController {
    private final StorageService service;
    private final JwtService jwtService;
    private final DirectoryService directoryService;
    private final FileService fileService;

    @GetMapping("/Info")
    StorageInfo getInfo(HttpServletRequest request) {
        long userId = jwtService.extractId(request);
        return service.info(userId);
    }

    @GetMapping(value = "/Download/{id}", produces = "application/octet-stream")
    @ResponseBody Resource download(@PathVariable long id, HttpServletRequest request, FileRequest fileRequest) {
        long userId = jwtService.extractId(request);
        File file = fileService.file(id);
        Directory directory = directoryService.directory(file.getDirectoryId());

        if (directory.getOwnerId() != userId && !file.isShared())
            throw new ResponseStatusException(
                    HttpStatus.FORBIDDEN,
                    "Access for file denied"
            );

        if (directory.getOwnerId() != userId && file.getPassword().isPresent()) {
            if (fileRequest.getPassword().isEmpty())
                throw new ResponseStatusException(
                        HttpStatus.FORBIDDEN,
                        "Password required"
                );

            if (!fileRequest.getPassword().get().equals(file.getPassword().get()))
                throw new ResponseStatusException(
                        HttpStatus.FORBIDDEN,
                        "Wrong password for file"
                );
        }

        return service.download(id); //todo ну вообще, метод должен называть upload, потому что для сервера это выгрузка файла, только для пользователя это загрузка
    }

    @PutMapping(path = "/Upload/{id}", consumes = "multipart/form-data")
    void upload(@PathVariable long id, HttpServletRequest request, @RequestParam("file") MultipartFile file) {
        //todo сдесь нужно сделать проверку есть ли у пользователя место на данные приколы
        long userId = jwtService.extractId(request);
        service.upload(id, file);
    }
}
